-----------------------------------------------------------------------------------------------------------
--[[
	NaW - Cursor Class
    Copyright (C) 2011.
	Written by Roland Yonaba - E-mail: roland[dot]yonaba[at]gmail[dot]com
	
		This file holds Map Class and metamethods
--]]
-----------------------------------------------------------------------------------------------------------
local setmetatable = setmetatable

local Cursor = {}

function Cursor:init()
	self.getX = love.mouse.getX
	self.getY = love.mouse.getY
	self.x = self:getX()
	self.y = self:getY()
	self.getPosition = love.mouse.getPosition
	self.setPosition = love.mouse.setPosition
	self.img = love.graphics.newImage('Images/Resources/Selector.png')
	self.w = 15
	self.h = 16
	self.w0,self.h0,self.x0,self.y0 = 0,0,0,0
	self.isDragging = false
	function self:isDown(...) return love.mouse.isDown(...) end
	love.mouse.setVisible(false)
	self.once = false
end

function Cursor:pressedOnce(...)
	if not self.once then
		if self:isDown(unpack(arg)) then
		self.once = true
		return true
		end
	end
	return false
end

function Cursor:isStaticHovering(item)
		if (self.x > item.x and self.x < item.x + item.w) and 
			(self.y > item.y and self.y < item.y + item.h) then 
		return true 
		end
	return false
end

function Cursor:isHovering(item,c_x,c_y)
		if (self.x > item.x - c_x) and (self.x < item.x + item.w - c_x) and 
			(self.y > item.y - c_y) and (self.y < item.y + item.h - c_y) then
		return true
		end
	return false
end

function Cursor:dragging(item)
	if (item.x > self.x0) and (item.x + item.w < self.x0 + self.w0) and
		(item.y > self.y0) and (item.y + item.h < self.y0 + self.h0) then
	return true
	end
	return false
end

function Cursor:draw(map)
	self.x = self:getX()
	self.y = self:getY()
		if self.isDragging then  
		self.w0 = self.x - self.x0 + map.mapX
		self.h0 = self.y - self.y0 + map.mapY
		love.graphics.rectangle('line',self.x0 - map.mapX + .5,self.y0 - map.mapY + .5,self.w0,self.h0)
		end
	love.graphics.draw(self.img, self.x, self.y)
end

function Cursor:onButtonPressed(x,y,button,map)
	if not self:isHovering(map.minimap,0,0) then
		self.isDragging = true
		self.x0 = self.x + map.mapX
		self.y0 = self.y + map.mapY
		self.w0,self.h0 = 0,0
	end
end

function Cursor:onButtonReleased(x,y,button)
	self.isDragging = false
	self.w0 = 0
	self.h0 = 0
	self.once = false
end

return setmetatable(Cursor, {__call = function(self,...) return self:init(...) end} )







