-----------------------------------------------------------------------------------------------------------
--[[
	NaW - Resources Manager
    Copyright (C) 2011.
	Written by Roland Yonaba - E-mail: roland[dot]yonaba[at]gmail[dot]com
		
		This file holds functions to manage Resources such as Food, Wood, Stone, Gold
--]]
-----------------------------------------------------------------------------------------------------------
local remove = table.remove
local insert = table.insert
local ceil = math.ceil
local setmetatable = setmetatable
local random = math.random
local ipairs = ipairs
local require  = require

local CONSTANTS = require 'Scripts/Constants'

local function removeCell(t,x,y)
local pos
	for k,v in ipairs(t) do
		if v.x == x and v.y == y then
		pos = k
		break
		end
	end
	if pos then 
	remove(t,pos) 
	end
end
	
local Resource = {}
Resource.__index = Resource
Resource.all = {}
Resource.img = {}
Resource.img.Food = {}
Resource.img.Gold = {}
Resource.img.Stone = {}
Resource.img.Wood = {}
	for i = 1,2 do table.insert(Resource.img.Food,love.graphics.newImage('Images/Resources/Fruits'..i..'.png')) end				   
	for i = 1,2 do table.insert(Resource.img.Stone,love.graphics.newImage('Images/Resources/Stone'..i..'.png')) end
	for i = 1,23 do table.insert(Resource.img.Wood,love.graphics.newImage('Images/Resources/Wood'..i..'.png')) end
					table.insert(Resource.img.Gold,love.graphics.newImage('Images/Resources/Gold1.png'))
Resource.stock	= { Food = 100, Wood = 200, Stone = 100, Gold = 75 }		   

function Resource:new(map,rem,type,x,y)
	local o = {type = type, selected = false}
	o.img = self.img[o.type][random(1,#self.img[o.type])]
	o.w = ceil(o.img:getWidth() / map.tileWidth)
	o.h = ceil(o.img:getHeight() / map.tileHeight)
	o.stock = self.stock[o.type]
		for i = y,y + o.h - 1 do
			for j = x,x + o.w - 1 do
				if map.cMap[i] and map.cMap[i][j] then map.cMap[i][j] = 1 end
				removeCell(rem,j,i)	
			end
		end		
	o.w = o.w * map.tileWidth
	o.h = o.h * map.tileHeight
	o.x = x * map.tileWidth - map.tileWidth
	o.y = y * map.tileHeight - map.tileHeight	
	insert(self.all,setmetatable(o,self))
end

function Resource:create(map,rem,n,type)
	for i = 1,n do
		local randmCell = random(1,#rem)
		local x,y = rem[randmCell].x, rem[randmCell].y
		self:new(map,rem,type,x,y)
	end 
end

function Resource:qCreate(map,nFood,nWood,nStone,nGold)
	local rem = {}
		for y in ipairs(map.cMap) do
			for x in ipairs(map.cMap[y]) do
			insert(rem,{x = x, y = y})
			end
		end
	self:create(map,rem,nFood,'Food')
	self:create(map,rem,nWood,'Wood')
	self:create(map,rem,nStone,'Stone')
	self:create(map,rem,nGold,'Gold')		
end

function Resource:isVisible(map)	
	if  (self.stock > 0) and ((map.mapX < self.x + self.w) and 
		(map.mapX + map.backgroundWidth > self.x) and 
		(map.mapY < self.y+self.h) and 
		(map.mapY + map.backgroundHeight > self.y)) then
		return true
	end
	return false
end

function Resource:deselectAll(button)
	if button == 'r' then
		for k in ipairs(self.all) do 
		self.all[k].selected = false 
		end
	end
end

function Resource:drawAll(map,cursor)
	for k,res in ipairs(self.all) do
		if cursor:dragging(res) then
		res.selected = true
		end
		if res:isVisible(map) then		
			if (cursor:isHovering(res,map.mapX,map.mapY) and cursor:isDown('l')) then 
			res.selected = true
			end	
			if res.selected then
			love.graphics.setColor(CONSTANTS.COLORS.RED)
			love.graphics.rectangle('fill',res.x - map.mapX, res.y - 10 - map.mapY, 30, 2)
			love.graphics.setColor(CONSTANTS.COLORS.GREEN)
			love.graphics.rectangle('fill',res.x - map.mapX, res.y - 10 - map.mapY, 30 * ceil(res.stock/self.stock[res.type]), 2)
			love.graphics.setColor(CONSTANTS.COLORS.WHITE)
			end
			love.graphics.draw(res.img,res.x - map.mapX,res.y - map.mapY)
		end
	end
end

return setmetatable(Resource, {__call = function(self,...) return self:qCreate(...) end} )